{% extends '!layout.html' %}
{% block extrahead %}
{{super()}}
<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/github-fork-ribbon-css/0.2.3/gh-fork-ribbon.min.css" />
{% endblock %}
{% block document %}
{{super()}}
    <a class="github-fork-ribbon" href="https://github.com/fabilab/iplotx" data-ribbon="Fork me on GitHub" title="Fork me on GitHub">Fork me on GitHub</a>
{% endblock %}
