"""
All artists defined in iplotx.
"""

from .network import NetworkArtist
from .tree import TreeArtist
from .vertex import VertexCollection
from .edge import EdgeCollection
from .label import LabelCollection
from .edge.arrow import EdgeArrowCollection
from .edge.leaf import LeafEdgeCollection
from .cascades import CascadeCollection
from .art3d.vertex import Vertex3DCollection
from .art3d.edge import Edge3DCollection


___all__ = (
    NetworkArtist,
    TreeArtist,
    VertexCollection,
    EdgeCollection,
    LeafEdgeCollection,
    LabelCollection,
    EdgeArrowCollection,
    CascadeCollection,
    Vertex3DCollection,
    Edge3DCollection,
)
